	
;simple non-updating scroll
	cpu 68000
	supmode on

	include "reg.inc"

;RAM
	org $100000
vblankflag:	ds.w 1	;signal for main/vblank processing
xscroll:	ds.w 1	;current xscroll position
yscroll:	ds.w 1	;current yscroll position

;ROM header
	org $0064
	dc.l VBLANK	;IRQ handler

	org $0100
	dc.b "NEO-GEO",$00

	org $0108
	dc.w $017	;Sengoku

	org $0122
	jmp Start	;entry

	org $0114
	dc.w $0100	;logo flag, don't show it just go straight to the entry point

	org $0182
	dc.l Code	;code pointer
Code:
	dc.l $76004A6D,$0A146600,$003C206D,$0A043E2D
	dc.l $0A0813C0,$00300001,$32100C01,$00FF671A
	dc.l $30280002,$B02D0ACE,$66103028,$0004B02D
	dc.l $0ACF6606,$B22D0AD0,$67085088,$51CFFFD4
	dc.l $36074E75,$206D0A04,$3E2D0A08,$3210E049
	dc.l $0C0100FF,$671A3010,$B02D0ACE,$66123028
	dc.l $0002E048,$B02D0ACF,$6606B22D,$0AD06708
	dc.l $588851CF,$FFD83607
	dc.w $4e75

;-----

;code:

Start:
	lea	VRAM_RW,A6	;A6 always has $3C0002
	jsr	ClearFIX	;no fix tiles on screen
	jsr	SetPalette
	jsr	SetSCB1
	jsr	SetSCB2
	jsr	SetSCB3
	and	#$F0FF,SR	;allow interrupts now
.loop
	move.w	xscroll,D0		;load xscroll to potentially modify
	move.w	yscroll,D1		;load yscroll ""
	move.b	BIOS_P1CURRENT,D2	;load controller data to test
.testLeft
	btst	#CNT_LE,D2	;scroll to the left?
	beq	.testRight
.left
	addq.w	#2,D0		;xscroll++
	bra	.testUp
.testRight
	btst	#CNT_RI,D2	;scroll to the right?
	beq	.testUp
.right
	subq.w	#2,D0		;xscroll--
.testUp
	btst	#CNT_UP,D2	;scroll up?
	beq	.testDown
.up
	addq.w	#2,D1		;yscroll++
	bra	.storeScroll
.testDown
	btst	#CNT_DN,D2	;scroll down?
	beq	.storeScroll
.down
	subq.w	#2,D1		;yscroll--
.storeScroll
	and.w	#$01FF,D0	;limit scroll to 9bits, but isn't essential
	move.w	D0,xscroll	;set scrollx for next vblank
	and.w	#$01FF,D1	;same for yscroll
	move.w	D1,yscroll
	st	vblankflag	;signal that main loop is processed
.spin
	tst.b	vblankflag	;wait until vblank happens
	bne	.spin
	bra	.loop

;---

ClearFIX:
	move.w	#FIXMAP,-2(A6)	;writing to FIX layer
	move.w	#1,2(A6)
	move.w	#$00FF,D0	;transparent tile
	move.w	#$500-1,D1	;clear entire map
.loop
	move.w	D0,(A6)
	dbra	D1,.loop
	rts

;---

SetPalette:
	clr.w	$400000		;black blanking color
	lea	.palette,A0	;pointer to palette in ROM
	move.w	(A0)+,$401FFE	;backdrop color is color 0 from palette
	lea	$400002,A1	;palette RAM target
	moveq	#15-1,D0	;15 colors remaining to copy
.loop
	move.w	(A0)+,(A1)+
	dbra	D0,.loop
	rts
.palette
	binclude "pal.bin"

;---

SetSCB1:
	move.w	#SCB1+64,-2(A6)	;write to SCB1 starting from sprite 1, NOT sprite 0
	move.w	#1,2(A6)
	move.w	#$0000,D1	;fixed high word
	moveq	#$0020,D4	;number of tiles per row, added between row updates	
	move.w	#$20-1,D3	;32 sprites to update
.loopOuter
	move.w	#$20-1,D2	;32 pairs of words for each sprite
	moveq	#$1F,D0		;initial tile # to store
	sub.w	D3,D0		;D3 is effectively 'sprite column counter', used to choose base tile # for this column
.loopInner
	move.w	D0,(A6)		;write sprite tile #
	add.w	D4,D0		;moving to next row, so offset tile #
	move.w	D1,(A6)		;write fixed high word ($0000)
	dbra	D2,.loopInner
.breakInner
	dbra	D3,.loopOuter
.breakOuter
	rts

;---

SetSCB2:
	move.w	#SCB2+1,-2(A6)	;writing to SCB2
	move.w	#1,2(A6)
	move.w	#$0FFF,D0	;X zoom max, Y zoom max
	move.w	#32-1,D1	;32 sprites are used in total
.loop
	move.w	D0,(A6)
	dbra	D1,.loop
	rts

;---

YPOS	SET $01F0	;sprites initially placed at top of screen
SIZE	SET $20		;sprites are 32 tiles or 512px tall

SetSCB3:
	move.w	#SCB3+1,-2(A6)		;writing to SCB3
	move.w	#1,2(A6)
	move.w	#YPOS << 7 | SIZE,(A6)	;set master sprite position and size
	move.w	#$0040,D0		;set chained sprites chain bit, automatically inherits other properties from master sprite
	move.w	#31-1,D1		;31 sprites are chained to master sprite
.loop
	move.w	D0,(A6)
	dbra	D1,.loop
	rts

;---

VBLANK:
	move.b	D0,$300001	;watchdog
	tst.b	$10FD80		;BIOS wants vblank?
	bmi	.regular
	jmp	$C00438
.regular
	move.w	#4,REG_IRQACK
	tst.b	vblankflag	;don't do anything if main routine hasn't finished yet
	beq	.return
.process
	movem.l	D0-D7/A0-A6,-(A7)		
	jsr	UpdateVRAM	;make VRAM changes needed for scrolling
	jsr	BIOSF_VBL	;read controller etc.
	movem.l	(A7)+,D0-D7/A0-A6
	clr.b	vblankflag	;signal that vblank is processed
.return
	rte

;---

UpdateVRAM:
	move.w	#SCB4+1,-2(A6)	;writing to xpos of master sprite
	move.w	xscroll,D0
	asl.w	#7,D0		;xpos is in upper 9 bits
	move.w	D0,(A6)

	move.w	#SCB3+1,-2(A6)	;writing to ypos of master sprite
	move.w	yscroll,D0
	neg.w	D0		;$01F0-Y on screen
	add.w	#$01F0,D0
	asl.w	#7,D0		;ypos is in upper 9 bits
	move.w	(A6),D1		;read current VRAM word
	and.w	#$007F,D1	;mask out old scroll bits
	or.w	D0,D1		;apply new scroll bits
	move.w	D1,(A6)		;write back to VRAM
	rts